__version__='1.1.0'

from .add_line_legend import add_line_legend
from .add_marker_legend import add_marker_legend
from .colname_stats import colname_stats
from .compare_directory import file_stats
from .get_file_directory import get_files_in_directory
from .get_time_series import get_time_series
from .get_time_series_axes import get_time_series_axes
from .get_time_series_plot_options import get_time_series_plot_options
from .is_list_of_strings import is_list_of_strings
from .plot_xy import plot_xy
from .plot_xy_axes import plot_xy_axes
from .plot_xy_line import plot_xy_line
from .plot_xy_markers import plot_xy_markers
from .read_element_data import read_element_data
from .TeX_code import TeX_code
from .time_series_plot import time_series_plot
